import { Component, OnInit } from '@angular/core';

import { ParamService } from '../../services/param/param.service';
import { HttpService } from '../../services/http/http.service';
import { InternationalisationService } from '../../services/internationalisation/internationalisation.service';
import { FieldSet } from '../../components/field-set/field-set.component';

@Component({
    selector: 'cond-distri',
    template: `
  <h1>{{_title}}</h1>
  <field-set *ngFor="let fs of _fieldSets" [fieldSet]=fs></field-set>
  <!--
  <field-set *ngFor="let fs of _fieldSets" title="fs.title" fields="fs.paramList"></field-set>
  <field-set title="titre de field set" fields="Q,D"></field-set>
  <param-input symbol="Q"></param-input>
  <param-input symbol="Q"></param-input>
  <param-input symbol="Ks"></param-input>
  <field-set title="aaaa"></field-set>
  -->
  `
})
export class CondDistriComponent implements OnInit {
    private _title = "Conduite distributrice";

    private _localisation = {};
    private _config = {};
    private _fieldSets: FieldSet[] = [];

    constructor(private paramService: ParamService, private httpService: HttpService, private intlService: InternationalisationService) {
    }

    private loadLocalisation() {
        let ths = this;
        let processData = function (s: string) {
            // fermeture nécessaire pour capturer la valeur de this (undefined sinon)
            ths._localisation = JSON.parse(s);
            ths.paramService.updateLocalisation(ths._localisation);
        }


        let f: string = "app/calculators/cond_distri/cond_distri." + this.intlService.languageCode + ".json"
        this.httpService.httpGetRequest(undefined, undefined, undefined, f, processData);
    }

    private logObject(obj: {}) {
        console.log(JSON.stringify(obj));
    }

    private parseConfig() {
        this._fieldSets = [];

        for (let conf_index in this._config) {
            //this.logObject(this._fieldSets);

            let conf = this._config[conf_index];
            //console.log(conf);
            let conf_id: string = conf["id"];
            if (conf_id.startsWith("fs_")) {
                let fieldSet: FieldSet = new FieldSet();
                fieldSet.title = this._localisation[conf_id];

                let fields = conf["fields"];
                for (let field_index in fields) {
                    let field = fields[field_index];
                    // console.log(field);
                    if (field["type"] === "input") {
                        let input_id = field["id"];
                        // console.log(input_id);
                        let param = this.paramService.getParameter(input_id);
                        if (param != undefined) {
                            //this.logObject(param);
                            fieldSet.params.push(param);
                        }
                    }
                }
                if (fieldSet.params.length > 0) {
                    // this.logObject(fieldSet);
                    this._fieldSets.push(fieldSet);
                }
            }
        }
    }

    private loadConfig() {
        let ths = this;
        let processData = function (s: string) {
            // fermeture nécessaire pour capturer la valeur de this (undefined sinon)
            ths._config = JSON.parse(s);
            ths.parseConfig();
        }

        let f: string = "app/calculators/cond_distri/cond_distri.config.json"
        this.httpService.httpGetRequest(undefined, undefined, undefined, f, processData);
    }

    ngOnInit() {
        this.loadLocalisation();
        this.loadConfig();
    }
}
